#<profile4xi profile = "02" />
# -------------------------------------------------------------------------------------------------
#<program4xi program = "listener4xi.py">
#<abstract4xi abstract = "provides the class Listener4Xi"/> 
#<refer4xi reference = "connection.py"/>
#<state4xi status = "ACK4tagging; ACK4moduletesting; NACK4systemtesting; [Rel 7.21]"/>
#<note4xi note = "__init__ of Listener4Xi changed for [Rel 7.21]"/>
# -------------------------------------------------------------------------------------------------

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'Listener4Xi' ] 
#"
#/>

# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import io
import os
import sys
import pickle
import select
import socket
import struct
import errno
import time
import tempfile
import itertools
try:
    import _winapi
    from _winapi import WAIT_OBJECT_0, WAIT_TIMEOUT, INFINITE
except ImportError:
    if sys.platform == 'win32':
        raise
    _winapi = None  
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing 
from multiprocessing import connection
from multiprocessing.connection import Listener
#"
#/>

# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from address_type4xi import address_type4xi
from answer_challenge4xi import answer_challenge4xi
from arbitrary_address4xi import arbitrary_address4xi

from constant_family4xi import ConstantFamily4Xi
from constant_system4xi import ConstantSystem4Xi

from deliver_challenge4xi import deliver_challenge4xi

from examine4xi import examine4xi

from pipelistener4xi import PipeListener4Xi
from print4xi import print4xi

from reason_object4xi import reason_object4xi

from sense_object4xi import sense_object4xi
from socketlistener4xi import SocketListener4Xi

from time4xi import time4xi
from _validate_family4xi import _validate_family4xi # is not used in __init__, just imported 
#"
#/>

# ==============================================================================================================
#<class4xi class = "Listener4Xi" base = "object" child = "None" synopsis = "returns a listener object" usage = "
#
#   BaseManager4Xi:
#       __init__:
#            set self._Listener to : <class 'listener4xi.Listener4Xi'>}
#   SyncManager4Xi:
#        __init__:
#            set self._Listener to : <class 'listener4xi.Listener4Xi'>; time: 23:40:15 09/06/09 FLE Daylight Time}
#
#   connection.py
#       class XmlListener(Listener):
#           accept(self):
#               obj = Listener.accept(self)
#"
#>
#<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
# ==============================================================================================================
class Listener4Xi(object):
    '''
    returns a listener object; a wrapper for a bound socket listening for connections, or for a Windows named pipe.
    '''
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi exposure = "
    _exposed_ = ('accept', 'close', 'sense', 'reason', 'ace')          
    #"
    #/>

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    _cf2xi = ConstantFamily4Xi()
    _cs2xi = ConstantSystem4Xi()
    _ack2listener = 0
    _nack2listener = 0
    
    
    address = property(lambda self: self._listener._address)
    last_accepted = property(lambda self: self._listener._last_accepted)

    
    #"
    #/>
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-Listener4Xi" usage = "initializes Listener4Xi">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __init__(self, address=None, family=None, backlog=1, authkey=os.urandom(32)):
        '''
        initializes Listener4Xi
        '''
                    
        # -----------------------------------------------------------------------------------------    
        # (Step 1/6)
        # -----------------------------------------------------------------------------------------         
        class2print = "Listener4Xi"
        method2print = "__init__"
        
        message2print = "{(Step 1/6a) started" + "; init-of-Listener4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 15:29:48 03/14/13 FLE Standard Time                
     
        message2print = "{(Step 1/6b) borning" + "; listener2xi-of-Listener4Xi-borning: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 15:29:48 03/14/13 FLE Standard Time
                   
        message2print = "{(Step 1/6c) inheriting" + "; object-inheriting-by-listener2xi-of-Listener4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'object'>
                        
        message2print = "{(Step 1/6d1) passed" + "; address-passed-to-init-of-Listener4Xi: " + str(address) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # None
        
        message2print = "{(Step 1/6d2) commenting" + "; address-parameter-commenting-by-init-of-Listener4Xi: address is reset even if a caller gives a value! " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #          
                     
        message2print = "{(Step 1/6e1) passed" + "; family-passed-to-init-of-Listener4Xi: " + str(family) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # None
        
        message2print = "{(Step 1/6e2) commenting" + "; family-parameter-commenting-by-init-of-Listener4Xi: family is reset even if a caller gives a value! " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
                       
        message2print = "{(Step 1/6f) passed" + "; backlog-passed-to-init-of-Listener4Xi: " + str(backlog) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 1
                     
        message2print = "{(Step 1/6g) passed" + "; authkey-passed-to-init-of-Listener4Xi: " + str(authkey) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # b'$\x8cR9\xf2\x1eVM\x1c\x90%\xa60\xddmh-\xa8\x05\xa9v{{\xa8\xa3o\x06\x9e:\xbc(\xce'; time: 15:29:48 03/14/13 FLE Standard Time}

        #                       
        if examine4xi(class2print, method2print):    
            # -----------------------------------------------------------------------------------------
            # StepOriginal:        
            # ----------------------------------------------------------------------------------------- 

            message2print = "{(StepOriginal) passing" + "; address-passing-to-Listener-by-init-of-Listener4Xi: " + str(address) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
      
            message2print = "{(StepOriginal) passing" + "; family-passing-to-Listener-by-init-of-Listener4Xi: " + str(family) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
      
            message2print = "{(StepOriginal) passing" + "; backlog-passing-to-Listener-by-init-of-Listener4Xi: " + str(backlog) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
      
            message2print = "{(StepOriginal) passing" + "; authkey-passing-to-Listener-by-init-of-Listener4Xi: " + str(authkey) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
                                        
            
            Listener.__init__(self, address, family, backlog, authkey)
            
            message2print = "{(StepOriginal) performed" + "; Original-Listener-performed-by-init-of-Listener4Xi: " + str(Listener.__init__) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
                                 
        else:
            # -------------------------------------------------------------------------------------    
            # (Step 2/6) - dragonized; forces family to be reasonable for Step 3/6
            # -------------------------------------------------------------------------------------         

            family = Listener4Xi._cf2xi.get_default_family()         #dragonized by forcing to the right family!
            
            message2print = "{(Step 2/6) set" + "; family-set-by-init-of-Listener4Xi to: " + str(family)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # AF_PIPE    
                    
            # -------------------------------------------------------------------------------------    
            # (Step 3/6)
            # -------------------------------------------------------------------------------------         
            
            address = address or arbitrary_address4xi(family) # if address is not given as parameter, an arbitrary address is being built
            
            message2print = "{(Step 3/6) set" + "; address-of-listener4xi-set-by-init-of-Listener4Xi to: " + str(address)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # \\.\pipe\pyc-2068-0-31n6kv
                         
                
            # -------------------------------------------------------------------------------------    
            # (Step 4/6) 
            # -------------------------------------------------------------------------------------         
            family2listener   = Listener4Xi._cf2xi.get_af_pipe()
            message2print = "{(Step 4a2/6) set" + "; family2listener-set-by-init-of-Listener4Xi to: " + str(family2listener)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # AF_PIPE
            
            plarform2listener = Listener4Xi._cs2xi.get_windows()
            message2print = "{(Step 4a3/6) set" + "; plarform2listener-set-by-init-of-Listener4Xi to: " + str(plarform2listener)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # win32
            
            if ((family == family2listener) or (sys.platform == plarform2listener)): #dragonized

                # -----------------------------------------------------------------------------------------    
                # Windows
                # -----------------------------------------------------------------------------------------     
                message2print = "{(Step 4b1/6) using" + "; family-using-by-init-of-Listener4Xi: " + str(family)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                # AF_PIPE
                
                message2print = "{(Step 4b2/6) using" + "; platform-using-by-init-of-Listener4Xi: " + str(sys.platform)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                # win32
                
                self._listener = PipeListener4Xi(address, backlog)
                
                message2print = "{(Step 4a1/6) set" + "; listener-of-listener4xi-set-by-init-of-Listener4Xi to: " + str(self._listener)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <pipelistener4xi.PipeListener4Xi object at 0x0000000003630E48>
                                
            else:

                # -----------------------------------------------------------------------------------------    
                # OSx
                # -----------------------------------------------------------------------------------------     
            
                message2print = "{(Step 4f/6) passing" + "; address-passing-to-SocketListener4Xi-by-init-of-Listener4Xi: " + str(address) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                message2print = "{(Step 4g/6) passing" + "; family-passing-to-SocketListener4Xi-by-init-of-Listener4Xi: " + str(family) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
                
                message2print = "{(Step 4h/6) passing" + "; backlog-passing-to-SocketListener4Xi-by-init-of-Listener4Xi: " + str(backlog) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)            
 
                try:
                    self._listener = SocketListener4Xi(address, family, backlog)
                
                
                    message2print = "{(Step 4i/6) set" + "; listener-of-listener2xi-set-by-init-of-Listener4Xi to: " + str(self._listener)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                    
                    Listener4Xi._ack2listener = Listener4Xi._ack2listener + 1
                    
                    message2print = "{(Step 4j/6) set" + "; ack2listener-set-by-init-of-Listener4Xi to: " + str(Listener4Xi._ack2listener)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)    
                    
                        
                except:
                    Listener4Xi._nack2listener = Listener4Xi._nack2listener + 1 
                                   
                    message2print = "{(Step 4z3/6) yielded" + "; nack2listener-yielded-by-init-of-Listener4Xi: " + str(nack2listener)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 

                               
                
            # -------------------------------------------------------------------------------------    
            # (Step 5/6) - dragonized both this step and default parameter setting
            # -------------------------------------------------------------------------------------         
    
            if authkey is None: 
                
                authkey=os.urandom(32) #dragonized, becomes an instance of bytes automatically
                
                message2print = "{(Step 5a/6) set" + "; authkey-set-by-init-of-Listener4Xi: " + str(authkey)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
    
            self._authkey = authkey
            
            message2print = "{(Step 5b/6) set" + "; authkey-of-listener2xi-set-by-init-of-Listener4Xi to: " + str(self._authkey)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # b'\xf4\x88\xff\x98\x8c\x1b\xfc\xc9|#-\xaa\x11\x84;\x00\xe0\xddK%d*\xe8\xf0v\x9b\xcb\x1b\x7fd\xa2\x02'
            
            # -------------------------------------------------------------------------------------    
            # (Step 6/6)
            # -------------------------------------------------------------------------------------         
            message2print = "{(Step 6a/6) inherited" + "; object-inherited-by-listener2xi-of-Listener4Xi: " + str(object)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <class 'object'>
                    
            message2print = "{(Step 6b/6) activated" + "; listener2xi-of-Listener4Xi-activated: " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 08:46:17 03/14/13 Pacific Daylight Time
            
            message2print = "{(Step 6c/6) returning" + "; init-of-Listener4Xi-returning: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 08:46:17 03/14/13 Pacific Daylight Time                                     
    #</methodize4xi>
             
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "accept-of-Listener4Xi" usage = "
    #   connection.py
    #       class XmlListener(Listener): 
    #           def accept(self): 
    #               obj = Listener.accept(self)
    #"> 
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def accept(self):
        '''
        Listener4Xi accept method 
        '''
                    
        # -----------------------------------------------------------------------------------------    
        # (Step 1/3)
        # -----------------------------------------------------------------------------------------         
    
        class2print = "Listener4Xi"
        method2print = "accept"
    
        message2print = "{(Step 1/6a) started" + "; accept-of-Listener4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 08:50:41 03/14/13 Pacific Daylight Time
        
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) returning" + "; Listener-accept-returning-by-accept-of-Listener4Xi: " + str(Listener.accept) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                           
            
            return Listener.accept(self)
             
        else:
            # -----------------------------------------------------------------------------------------
            # (Step 2/3)      
            # -----------------------------------------------------------------------------------------         
            ack2connection = False
            nack2connection = True
            
            connection2xi = None
            
            try:                               
                connection2xi = self._listener.accept()
                
                message2print = "{(Step 2/3a1) set" + "; connection2xi-set-by-accept-of-Listener4Xi to: " + str(connection2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                   
            
                ack2connection = True
                
                message2print = "{(Step 2/3a2) set" + "; ack2connection-set-by-accept-of-Listener4Xi to: " + str(ack2connection)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            except:
                message2print = "{(Step 2/3a3) yielded" + "; nack2connection-yielded-by-accept-of-Listener4Xi: " + str(nack2connection)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
            
            # -----------------------------------------------------------------------------------------
            # 
            # -----------------------------------------------------------------------------------------                      
       
            message2print = "{(Step 2/3b) analyzing" + "; authkey-of-listener2xi-analyzing-by-accept-of-Listener4Xi: " + str(self._authkey)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            
            if self._authkey:
                
                # -----------------------------------------------------------------------------------------
                # Authorization established
                # -----------------------------------------------------------------------------------------                      
                ack2challenge = False
                nack2challenge = True                            
                    
                if ack2connection is True:
                    # -------------------------------------------------------------------------------------
                    # Connection exists
                    # -------------------------------------------------------------------------------------                 
                    ack2deliver = False
                    nack2deliver = True                    
                    try:
                        # ---------------------------------------------------------------------------------
                        # Delivering challenge to connection
                        # ---------------------------------------------------------------------------------                      
                        message2print = "{(Step 2/3c) passing" + "; connection2xi-passing-to-deliver-challenge-by-accept-of-Listener4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)            
                        
                        message2print = "{(Step 2/3d) passing" + "; authkey-of-listener2xi-passing-to-deliver-challenge-by-accept-of-Listener4Xi: " + str(self._authkey) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)            
                        
                        deliver_challenge4xi(connection2xi, self._authkey)
                        
                        message2print = "{(Step 2/3e) performed" + "; deliver-challenge-performed-by-accept-of-Listener4Xi: " + str(deliver_challenge4xi) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        
                        ack2deliver = True
                        
                        message2print = "{(Step 2/3f) set" + "; ack2deliver-set-by-accept-of-Listener4Xi to: " + str(ack2deliver)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)    
                    except:
                        # ---------------------------------------------------------------------------------
                        # 
                        # ---------------------------------------------------------------------------------                      
                        message2print = "{(Step 2/3g) yielded" + "; nack2deliver-yielded-by-accept-of-Listener4Xi: " + str(nack2deliver)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 

                    ack2answer = False
                    nack2answer = True                    
                    try:    
                        # ---------------------------------------------------------------------------------
                        # 
                        # ---------------------------------------------------------------------------------                    
                        message2print = "{(Step 2/3h) passing" + "; connection2xi-passing-to-answer-challenge4xi-by-accept-of-Listener4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        
                        message2print = "{(Step 2/3i) passing" + "; authkey-of-listener2xi-passing-to-answer-challenge4xi-by-accept-of-Listener4Xi: " + str(self._authkey) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)             
                  
                        
                        answer_challenge4xi(connection2xi, self._authkey)
                        
                        
                        message2print = "{(Step 2/3j) performed" + "; answer-challenge4xi-performed-by-accept-of-Listener4Xi: " + str(answer_challenge4xi) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        
                        ack2answer = True
                        
                        message2print = "{(Step 2/3k) set" + "; ack2answer-set-by-accept-of-Listener4Xi to: " + str(ack2answer)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                         
                    except:
                        # ---------------------------------------------------------------------------------
                        # 
                        # ---------------------------------------------------------------------------------                      
                        message2print = "{(Step 2/3l) yielded" + "; nack2answer-yielded-by-accept-of-Listener4Xi: " + str(nack2answer)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                         
                         
                else:
                    # -------------------------------------------------------------------------------------
                    # 
                    # -------------------------------------------------------------------------------------                  
                    message2print = "{(Step 2/3i) yielded" + "; nack2challenge-yielded-by-accept-of-Listener4Xi: " + str(nack2challenge)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                   
            
            # -----------------------------------------------------------------------------------------
            # 
            # -----------------------------------------------------------------------------------------                      
                
            else:
            
                # -------------------------------------------------------------------------------------
                # no authorization provided by caller
                # -------------------------------------------------------------------------------------                      
                   
                message2print = "{(Step 2/3z) yielded" + "; authkey-of-listener2xi-yielded-by-accept-of-Listener4Xi: " + str(self._authkey)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
            
                                            
            # -----------------------------------------------------------------------------------------
            # (Step 3/3)      
            # -----------------------------------------------------------------------------------------                
            message2print = "{(Step 3/3) returning" + "; connection2xi-returning-by-accept-of-Listener4Xi: " + str(connection2xi) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                                   
            
            return connection2xi
    #</methodize4xi>
 
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "close-of-Listener4Xi" usage = "closes the bound socket or named pipe">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def close(self):
        '''
        Listener4Xi close method closes the bound socket or named pipe of `self`.
        '''
                
        # -----------------------------------------------------------------------------------------    
        # (Step 1/2) 
        # -----------------------------------------------------------------------------------------         
         
     
        class2print = "Listener4Xi"
        method2print = "close"
        
        message2print = "{(Step 1/2a) started" + "; close-of-Listener4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                       
        
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) returning" + "; Listener-close-returning-by-close-of-Listener4Xi: " + str(Listener.close) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                          
            return Listener.close(self)
             
        else:
            # -----------------------------------------------------------------------------------------
            # (Step 2/2)     
            # -----------------------------------------------------------------------------------------         
            message2print = "{(Step 2/2) returning" + "; listener-close-of-listener2xi-returning-by-close-of-Listener4Xi: " + str(self._listener.close) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                    
            # { Listener4Xi : close ; returning by calling self._listener.close()time: 19:02:49 06/23/09 FLE Daylight Time}        
                                       
            return self._listener.close()
    #</methodize4xi>
        
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "__enter__-of-Listener4Xi" usage = "returns self">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def __enter__(self):
        '''
        returns self
        '''
                
        # -----------------------------------------------------------------------------------------    
        # (Step 1/2) 
        # -----------------------------------------------------------------------------------------         
        class2print = "Listener4Xi"
        method2print = "__enter__"
        
        message2print = "{(Step 1/2a) started" + "; __enter__-of-Listener4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                       
        
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) returning" + "; Listener-__enter__-returning-by-__enter__-of-Listener4Xi: " + str(Listener.__enter__) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                          
            return Listener.__enter__(self)
             
        else:
            # -----------------------------------------------------------------------------------------
            # (Step 2/2)     
            # -----------------------------------------------------------------------------------------         
            message2print = "{(Step 6/6c) returning" + "; init-of-Listener4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)       
                                       
            return self
            
    #</methodize4xi>    

            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "_exit__-of-Listener4Xi" usage = "closes">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def _exit__(self):
        '''
        closes
        '''
                
        # -----------------------------------------------------------------------------------------    
        # (Step 1/2) 
        # -----------------------------------------------------------------------------------------         
        class2print = "Listener4Xi"
        method2print = "_exit__"
        
        message2print = "{(Step 1/2a) started" + "; _exit__-of-Listener4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                       
        
        if examine4xi(class2print, method2print):
            # -----------------------------------------------------------------------------------------
            # (StepOriginal):        
            # ----------------------------------------------------------------------------------------- 
            message2print = "{(StepOriginal) returning" + "; Listener-_exit__-returning-by-_exit__-of-Listener4Xi: " + str(Listener._exit__) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)    
                          
            return Listener._exit__(self)
             
        else:
            # -----------------------------------------------------------------------------------------
            # (Step 2/2)     
            # -----------------------------------------------------------------------------------------         
                                         
            self.close()
            
            message2print = "{(Step 2a/2) performed" + "; close-performed-by-_exit__-of-Listener4Xi: " + str(self.close) + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
  
            message2print = "{(Step 2b/2) returning" + "; _exit__-of-Listener4Xi-returning: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            
            return            
                      
    #</methodize4xi>        
                                            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-Listener4Xi" usage = "senses Listener4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses Listener4Xi class objects
        '''
         
        class2print = 'Listener4Xi'
        method2print = 'sense'
        
     
        message2print = "{started" + "; sense-of-Listener4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
           
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        message2print = "{sensing" + "; Listener4Xi-sensing-by-sense-of-Listener4Xi: " + str(Listener4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
                   
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb2xi = sense_object4xi(self)
        
        message2print = "{got" + "; kb2xi-got-by-sense-of-Listener4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['listener-of-listener2xi'] = self._listener    
            kb2xi['authkey-of-listener2xi'] = self._authkey  
            kb2xi['listener-accept-of-listener2xi'] = self._listener.accept  
            kb2xi['listener-close-of-listener2xi'] = self._listener.close 
            kb2xi['ack2listener-of-Listener4Xi'] = Listener4Xi._ack2listener 
            kb2xi['nack2listener-of-Listener4Xi'] = Listener4Xi._nack2listener                         
      

        except:
            message2print = "{yielded" + "; kb2xi-yielded-by-sense-of-Listener4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            

        message2print = "{sensed" + "; Listener4Xi-sensed-by-sense-of-Listener4Xi: " + str(Listener4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        message2print = "{returning" + "; kb2xi-returning-by-sense-of-Listener4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                               
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
        
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-Listener4Xi" usage = "reasons Listener4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons Listener4Xi class objects
        '''
     
        
        class2print = 'Listener4Xi'
        method2print = 'reason'

     
        message2print = "{started" + "; reason-of-Listener4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
     
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{reasoning" + "; Listener4Xi-reasoning-by-reason-of-Listener4Xi: " + str(Listener4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
            
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{got" + "; kb4xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # -----------------------------------------------------------------------------------------          
        try:
            kb4xi['function-of-listener2xi'] = str(self._listener)    
            kb4xi['attribute-of-listener2xi'] = str(self._authkey)  
        except:
            message2print = "{yielded" + "; kb4xi-yielded-by-reason-of-Listener4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            

        message2print = "{reasoned" + "; Listener4Xi-reasoned-by-reason-of-Listener4Xi: " + str(Listener4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb4xi-returning-by-reason-of-Listener4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                            
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
        
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-Listener4Xi" usage = "aces Listener4Xi class objects">
    #<state4xi status = "ACK4review; ACK4tagging; NACK4moduletesting; NACK4systemtesting"/>
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces Listener4Xi class objects 
        '''
   
        
        
        class2print = 'Listener4Xi'
        method2print = 'ace'
        
     
        message2print = "{started" + "; ace-of-Listener4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
          
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        message2print = "{acing" + "; Listener4Xi-acing-by-ace-of-Listener4Xi: " + str(Listener4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
            
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{got" + "; kb8xi-got-by-ace-of-Listener4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
            


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # -----------------------------------------------------------------------------------------              
        try:
            kb8xi['number-of-functions-of-listener2xi'] = str(1)    
            kb8xi['number-of-attributes-of-listener2xi'] = str(1)          
        except:
            message2print = "{yielded" + "; kb8xi-yielded-by-ace-of-Listener4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
           
                  
        message2print = "{aced" + "; Listener4Xi-aced-by-ace-of-Listener4Xi: " + str(Listener4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)       
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{returning" + "; kb8xi-returning-by-ace-of-Listener4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)    
                            
            
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------                
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   test_none4xi.txt
#   sense-test-none4xi.xml
#   reason-test-none4xi.xml
#   ace-test-none4xi.xml
#"
#>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   references = "Legal notice at www.dragonxi.com; www.yourdragonxi.com; www.yourdragonxi.org; www.dragon2xi.com/; www.dragon4xi.com/; www.dragon8xi.com/ " 
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>
 

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of listener4xi.py                             
# ------------------------------------------------------------------------------------------------- 
#"
#/>


